#include <time.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <unistd.h>

#define PREALLOC	10
#define XLOAD_BIN	"/opt/pvmbin/xload"
#define GRANULARITY	2
#define RSHCMD		"rsh"

time_t astart;

struct tim_table{
  time_t start;
  time_t duration;
  int load;
  char node[128];
};

struct tim_table *table=NULL;
int table_size=0;
int table_pos=0;

int readconf(char *name){
  FILE *in;
  
  time_t start;
  time_t duration;
  int load;
  char node[128];
  
  char buffer[1024];
  
  if((in=fopen(name,"r"))==NULL){
    printf("unable to open input file: %s\n",name);
    exit(1);
  }
  
  while(fgets(buffer,sizeof(buffer),in)!=NULL){
    if(buffer[0]!='#'){
      if(sscanf(buffer,"%u %u %u %s",&start,&duration,&load,node)<4){
        printf("error recognizing input line: %s\n",buffer);
        exit(1);
      }
      if(table_pos>=table_size){
	table_size+=PREALLOC;
	table=(struct tim_table*)realloc(table,sizeof(struct tim_table)*table_size);
      }
      table[table_pos].start=start;
      table[table_pos].duration=duration;
      table[table_pos].load=load;
      strncpy(table[table_pos].node,node,128);
      
      table_pos++;
    }
  }
  
  fclose(in);
}

void fireup(int pos){
  int p;
  char duration[32];
  char load[32];


  p=fork();
  if(p!=0)return;
  
  sprintf(duration,"%u",table[pos].duration);
  sprintf(load,"%u",table[pos].load);

  // this is the child, let it execute the xload program ...
  printf("EXEC time: %u %s on %s [load:%s,durat:%s]\n",time(NULL)-astart,XLOAD_BIN,table[pos].node,load,duration);
  if (execlp (RSHCMD,table[pos].node,XLOAD_BIN,load,duration,NULL)<0){
    printf("error executing remote xload\n");
    exit(1);
  }
  exit(0);
}


int main(int argc,char *argv[]){
  int i;
  time_t now;
  int pos=0;
  
  astart=time(NULL);
  
  if(argc<2){
    printf("usage: %s <input_file>\n",argv[0]);
    exit(1);
  }

  readconf(argv[1]);

  printf("input   : %s\n",argv[1]);
  printf("entries : %i\n",table_pos);
  
  for(i=0;i<table_pos;i++){
    printf("start: %5u durat: %5u load: %2u node: %s\n",table[i].start,table[i].duration,table[i].load,table[i].node);
  }

  while(pos<table_pos){
    now=time(NULL);
    while((astart+table[pos].start)<=now){
      fireup(pos);
      pos++;
    }
    sleep(GRANULARITY);
  }
  return 0;
}
